/**
 * Creates and AT Internet SmartTag and dispatches it.
 */
(function ($, Drupal, once, drupalSettings) {

  // Process only if atsmarttag settings have been defined.
  if (drupalSettings.atsmarttag === undefined) {
    return;
  }

  Drupal.atsmarttag = {

    _instantiateTag(settings) {
      let tag_properties = {};
      if (!!settings.site) {
        tag_properties.site = settings.site;
      }
      if (!!settings.collect_domain) {
        tag_properties.collectDomain = settings.collect_domain;
      }
      if (!!settings.collect_domain_ssl) {
        tag_properties.collectDomainSSL = settings.collect_domain_ssl;
      }
      if (settings.secure !== 0) {
        tag_properties.secure = true;
      }
      if (settings.disable_cookie !== 0) {
        tag_properties.disableCookie = true;
      }
      if (!!settings.cookie_domain) {
        tag_properties.cookieDomain = settings.cookie_domain;
      }
      window.ATTag = new window.ATInternet.Tracker.Tag(tag_properties);
    },

    _configurePrivacy(settings) {
      if (window.ATTag.privacy) {
        if (settings.cnil_exempt !== 0) {
          window.ATTag.privacy.setVisitorMode('cnil', 'exempt');
        }
      }
    },

    _addChaptersToProperties(settings, properties) {
      if (!!settings.page_chapter1) {
        properties['chapter1'] = settings.page_chapter1;
      }
      if (!!settings.page_chapter2) {
        properties['chapter2'] = settings.page_chapter2;
      }
      if (!!settings.page_chapter3) {
        properties['chapter3'] = settings.page_chapter3;
      }
    },

    _processPage(settings) {
      let page_properties = {};
      if (!!settings.page_name) {
        page_properties.name = settings.page_name;
      }
      this._addChaptersToProperties(settings, page_properties);
      if (Object.getOwnPropertyNames(page_properties).length > 0) {
        window.ATTag.page.set(page_properties);
        window.ATTag.dispatch();
      }
    },

    _triggerInitializedEvent() {
      $(document).ready(function () {
        $(document).trigger('atsmarttag:tag_initialized');
      });
    },

    createTagAndDispatch(settings = drupalSettings.atsmarttag) {
      this._instantiateTag(settings);
      this._configurePrivacy(settings);
      this._processPage(settings);
      this._triggerInitializedEvent();
    },

    /**
     * Check whether this is a download URL or not.
     *
     * @param {string} url
     *   The web url to check.
     *
     * @return {boolean} isDownload
     */
    isDownload(url) {
      var isDownload = new RegExp('\\.(' + drupalSettings.atsmarttag.trackDownloadExtensions + ')([\?#].*)?$', 'i');
      return isDownload.test(url);
    },

    /**
     * Check whether this is an absolute internal URL or not.
     *
     * @param {string} url
     *   The web url to check.
     *
     * @return {boolean} isInternal
     */
    isInternal(url) {
      var isInternal = new RegExp('^(https?):\/\/' + window.location.host, 'i');
      return isInternal.test(url);
    },

  };

  Drupal.behaviors.atsmarttag = {
    attach: function(context, settings) {
      once('atsmarttag', 'html', context).forEach(function (element) {
        Drupal.atsmarttag.createTagAndDispatch(settings.atsmarttag);
      });
    },
  };

  $(document).ready(function () {

    // Attach mousedown, keyup, touchstart events to document only and catch
    // clicks on all elements.
    $(document.body).on('mousedown keyup touchstart', function (event) {

      // Catch the closest surrounding link of a clicked element.
      $(event.target).closest('a,area').each(function () {

        // Is the clicked URL internal?
        if (Drupal.atsmarttag.isInternal(this.href)) {
          // Is download tracking activated and the file extension configured
          // for download tracking?
          if (drupalSettings.atsmarttag.trackDownload && Drupal.atsmarttag.isDownload(this.href)) {
            // Download link clicked.
            let click_properties = {
              'elem': this,
              'name': this.href,
              'type': 'download',
            };
            Drupal.atsmarttag._addChaptersToProperties(
              drupalSettings.atsmarttag, click_properties);
            window.ATTag.click.send(click_properties);
          }
        }
        else {
          if (drupalSettings.atsmarttag.trackMailto && $(this).is("a[href^='mailto:'],area[href^='mailto:']")) {
            // Mailto link clicked.
            let click_properties = {
              'elem': this,
              'name': this.href,
              'type': 'action',
            };
            Drupal.atsmarttag._addChaptersToProperties(
              drupalSettings.atsmarttag, click_properties);
            window.ATTag.click.send(click_properties);
          }
          else if (drupalSettings.atsmarttag.trackOutbound && this.href.match(/^\w+:\/\//i)) {
            // External link clicked / No top-level cross domain clicked.
            let click_properties = {
              'elem': this,
              'name': this.href,
              'type': 'navigation',
            };
            Drupal.atsmarttag._addChaptersToProperties(
              drupalSettings.atsmarttag, click_properties);
            window.ATTag.click.send(click_properties);
          }
        }
      });
    });
  });

})(jQuery, Drupal, once, drupalSettings);
